#include "first_try.hpp"

#include "dynexite_exam.hpp"
#include "student_data.hpp"

#include <stdexcept>
#include <string>

#define ALLOW_MISSING_STUDENT_INFO 1

void FirstTry::recalculate() {

    const auto &examRef = std::get<const DynexiteExam &>(this->mParentRefs);
    const auto &studentRef = std::get<const StudentData &>(this->mParentRefs);

    mFirstTryClosedPoints.clear();

    // add the closed points of every student to the list, for whom this is
    // their "zeroth" / first try
    for (qsizetype studentIdx = 0;
         studentIdx < studentRef.mRegistrationNr.size(); ++studentIdx) {

        if (studentRef.mAttemptNr[studentIdx] <= 1) {

            // Get the array index for the same student in the exam file
            const auto examIdx = examRef.mIdentifier.indexOf(
                studentRef.mRegistrationNr[studentIdx]);

            // If we didn't find them:
            if (examIdx == -1) {
                /* TODO: what to do with students who don't appear in the
                 * RWTHOnline file and for which we therefore don't know the
                 * attempt number? */

#if ALLOW_MISSING_STUDENT_INFO
                continue;
#else
                throw std::invalid_argument(
                    "Didn't find student data for student with id " +
                    std::to_string(studentRef.mRegistrationNr[studentIdx]));
#endif
            }

            // Add their points to the right list
            if (studentRef.mAttemptNr[studentIdx] == 1) {
                mFirstTryClosedPoints.emplace_back(
                    examRef.mPointsWithoutBonus[GradingType::Closed][examIdx]);
            } else if (studentRef.mAttemptNr[studentIdx] == 0) {
                mZerothTryClosedPoints.emplace_back(
                    examRef.mPointsWithoutBonus[GradingType::Closed][examIdx]);
            } else {
                throw std::invalid_argument(
                    "Unexpected student attempt number: " +
                    std::to_string(studentRef.mAttemptNr[studentIdx]));
            }
        }
    }
}
